# coding: utf-8

"""
    Hangar API

    This page describes the format for the current Hangar REST API as well as general usage guidelines.<br> Note that all routes **not** listed here should be considered **internal**, and can change at a moment's notice. **Do not use them**.  ## Authentication and Authorization There are two ways to consume the API: Authenticated or anonymous.  ### Anonymous When using anonymous authentication, you only have access to public information, but you don't need to worry about creating and storing an API key or handing JWTs.  ### Authenticated If you need access to non-public content or actions, you need to create and use API keys. These can be created by going to the API keys page via the profile dropdown or by going to your user page and clicking on the key icon.  API keys allow you to impersonate yourself, so they should be handled like passwords. **Do not share them with anyone else!**  #### Getting and Using a JWT Once you have an API key, you need to authenticate yourself: Send a `POST` request with your API key identifier to `/api/v1/authenticate?apiKey=yourKey`. The response will contain your JWT as well as an expiration time. Put this JWT into the `Authorization` header of every request and make sure to request a new JWT after the expiration time has passed.  Please also set a meaningful `User-Agent` header. This allows us to better identify loads and needs for potentially new endpoints.  ## Misc ### Date Formats Standard ISO types. Where possible, we use the [OpenAPI format modifier](https://swagger.io/docs/specification/data-models/data-types/#format).  ### Rate Limits and Caching The default rate limit is set at 20 requests every 5 seconds with an initial overdraft for extra leniency. Individual endpoints, such as version creation, may have stricter rate limiting.  If applicable, always cache responses. The Hangar API itself is cached by CloudFlare and internally.

    The version of the OpenAPI document: 1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import re  # noqa: F401
import io
import warnings

from pydantic import validate_call, ValidationError
from typing import Dict, List, Optional, Tuple

from pydantic import Field
from typing_extensions import Annotated
from pydantic import StrictStr

from typing import List, Optional

from hangar_api.models.named_permission import NamedPermission
from hangar_api.models.permission_check import PermissionCheck
from hangar_api.models.user_permissions import UserPermissions

from hangar_api.api_client import ApiClient
from hangar_api.api_response import ApiResponse
from hangar_api.exceptions import (  # noqa: F401
    ApiTypeError,
    ApiValueError
)


class PermissionsApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client

    @validate_call
    def has_all(self, permissions : Annotated[List[NamedPermission], Field(min_length=0, max_length=50, description="The permissions to check")], slug : Annotated[Optional[StrictStr], Field(description="The project slug of the project to check permissions in. Must not be used together with `organizationName`")] = None, organization : Annotated[Optional[StrictStr], Field(description="The organization to check permissions in. Must not be used together with `projectOwner` and `projectSlug`")] = None, **kwargs) -> PermissionCheck:  # noqa: E501
        """Checks whether you have all the provided permissions  # noqa: E501

        Checks whether you have all the provided permissions in the given context  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.has_all(permissions, slug, organization, async_req=True)
        >>> result = thread.get()

        :param permissions: The permissions to check (required)
        :type permissions: List[NamedPermission]
        :param slug: The project slug of the project to check permissions in. Must not be used together with `organizationName`
        :type slug: str
        :param organization: The organization to check permissions in. Must not be used together with `projectOwner` and `projectSlug`
        :type organization: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: PermissionCheck
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            message = "Error! Please call the has_all_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return self.has_all_with_http_info(permissions, slug, organization, **kwargs)  # noqa: E501

    @validate_call
    def has_all_with_http_info(self, permissions : Annotated[List[NamedPermission], Field(min_length=0, max_length=50, description="The permissions to check")], slug : Annotated[Optional[StrictStr], Field(description="The project slug of the project to check permissions in. Must not be used together with `organizationName`")] = None, organization : Annotated[Optional[StrictStr], Field(description="The organization to check permissions in. Must not be used together with `projectOwner` and `projectSlug`")] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """Checks whether you have all the provided permissions  # noqa: E501

        Checks whether you have all the provided permissions in the given context  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.has_all_with_http_info(permissions, slug, organization, async_req=True)
        >>> result = thread.get()

        :param permissions: The permissions to check (required)
        :type permissions: List[NamedPermission]
        :param slug: The project slug of the project to check permissions in. Must not be used together with `organizationName`
        :type slug: str
        :param organization: The organization to check permissions in. Must not be used together with `projectOwner` and `projectSlug`
        :type organization: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(PermissionCheck, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'permissions',
            'slug',
            'organization'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method has_all" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats: Dict[str, str] = {}

        # process the path parameters
        _path_params: Dict[str, str] = {}

        # process the query parameters
        _query_params: List[Tuple[str, str]] = []
        if _params.get('permissions') is not None:  # noqa: E501
            _query_params.append(('permissions', _params['permissions']))
            _collection_formats['permissions'] = 'multi'

        if _params.get('slug') is not None:  # noqa: E501
            _query_params.append(('slug', _params['slug']))

        if _params.get('organization') is not None:  # noqa: E501
            _query_params.append(('organization', _params['organization']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, str] = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings: List[str] = ['HangarAuth']  # noqa: E501

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "PermissionCheck",
            '401': "PermissionCheck",
            '400': "PermissionCheck",
        }

        return self.api_client.call_api(
            '/api/v1/permissions/hasAll', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_call
    def has_any(self, permissions : Annotated[List[NamedPermission], Field(description="The permissions to check")], slug : Annotated[Optional[StrictStr], Field(description="The slug of the project to check permissions in. Must not be used together with `organizationName`")] = None, organization : Annotated[Optional[StrictStr], Field(description="The organization to check permissions in. Must not be used together with `projectOwner` and `projectSlug`")] = None, **kwargs) -> PermissionCheck:  # noqa: E501
        """Checks whether you have at least one of the provided permissions  # noqa: E501

        Checks whether you have at least one of the provided permissions in the given context  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.has_any(permissions, slug, organization, async_req=True)
        >>> result = thread.get()

        :param permissions: The permissions to check (required)
        :type permissions: List[NamedPermission]
        :param slug: The slug of the project to check permissions in. Must not be used together with `organizationName`
        :type slug: str
        :param organization: The organization to check permissions in. Must not be used together with `projectOwner` and `projectSlug`
        :type organization: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: PermissionCheck
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            message = "Error! Please call the has_any_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return self.has_any_with_http_info(permissions, slug, organization, **kwargs)  # noqa: E501

    @validate_call
    def has_any_with_http_info(self, permissions : Annotated[List[NamedPermission], Field(description="The permissions to check")], slug : Annotated[Optional[StrictStr], Field(description="The slug of the project to check permissions in. Must not be used together with `organizationName`")] = None, organization : Annotated[Optional[StrictStr], Field(description="The organization to check permissions in. Must not be used together with `projectOwner` and `projectSlug`")] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """Checks whether you have at least one of the provided permissions  # noqa: E501

        Checks whether you have at least one of the provided permissions in the given context  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.has_any_with_http_info(permissions, slug, organization, async_req=True)
        >>> result = thread.get()

        :param permissions: The permissions to check (required)
        :type permissions: List[NamedPermission]
        :param slug: The slug of the project to check permissions in. Must not be used together with `organizationName`
        :type slug: str
        :param organization: The organization to check permissions in. Must not be used together with `projectOwner` and `projectSlug`
        :type organization: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(PermissionCheck, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'permissions',
            'slug',
            'organization'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method has_any" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats: Dict[str, str] = {}

        # process the path parameters
        _path_params: Dict[str, str] = {}

        # process the query parameters
        _query_params: List[Tuple[str, str]] = []
        if _params.get('permissions') is not None:  # noqa: E501
            _query_params.append(('permissions', _params['permissions']))
            _collection_formats['permissions'] = 'multi'

        if _params.get('slug') is not None:  # noqa: E501
            _query_params.append(('slug', _params['slug']))

        if _params.get('organization') is not None:  # noqa: E501
            _query_params.append(('organization', _params['organization']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, str] = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings: List[str] = ['HangarAuth']  # noqa: E501

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "PermissionCheck",
            '401': "PermissionCheck",
            '400': "PermissionCheck",
        }

        return self.api_client.call_api(
            '/api/v1/permissions/hasAny', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_call
    def show_permissions(self, slug : Annotated[Optional[StrictStr], Field(description="The slug of the project get the permissions for. Must not be used together with `organizationName`")] = None, organization : Annotated[Optional[StrictStr], Field(description="The organization to check permissions in. Must not be used together with `projectOwner` and `projectSlug`")] = None, **kwargs) -> UserPermissions:  # noqa: E501
        """Returns your permissions  # noqa: E501

        Returns a list of permissions you have in the given context  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.show_permissions(slug, organization, async_req=True)
        >>> result = thread.get()

        :param slug: The slug of the project get the permissions for. Must not be used together with `organizationName`
        :type slug: str
        :param organization: The organization to check permissions in. Must not be used together with `projectOwner` and `projectSlug`
        :type organization: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: UserPermissions
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            message = "Error! Please call the show_permissions_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return self.show_permissions_with_http_info(slug, organization, **kwargs)  # noqa: E501

    @validate_call
    def show_permissions_with_http_info(self, slug : Annotated[Optional[StrictStr], Field(description="The slug of the project get the permissions for. Must not be used together with `organizationName`")] = None, organization : Annotated[Optional[StrictStr], Field(description="The organization to check permissions in. Must not be used together with `projectOwner` and `projectSlug`")] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """Returns your permissions  # noqa: E501

        Returns a list of permissions you have in the given context  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.show_permissions_with_http_info(slug, organization, async_req=True)
        >>> result = thread.get()

        :param slug: The slug of the project get the permissions for. Must not be used together with `organizationName`
        :type slug: str
        :param organization: The organization to check permissions in. Must not be used together with `projectOwner` and `projectSlug`
        :type organization: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(UserPermissions, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'slug',
            'organization'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method show_permissions" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats: Dict[str, str] = {}

        # process the path parameters
        _path_params: Dict[str, str] = {}

        # process the query parameters
        _query_params: List[Tuple[str, str]] = []
        if _params.get('slug') is not None:  # noqa: E501
            _query_params.append(('slug', _params['slug']))

        if _params.get('organization') is not None:  # noqa: E501
            _query_params.append(('organization', _params['organization']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, str] = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings: List[str] = ['HangarAuth']  # noqa: E501

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "UserPermissions",
            '401': "UserPermissions",
            '400': "UserPermissions",
        }

        return self.api_client.call_api(
            '/api/v1/permissions', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))
